'From SqueakNihongo6.1 of 17 April 2004 [latest update: #1] on 7 July 2004 at 9:58:49 am'!
Model subclass: #Scamper
	instanceVariableNames: 'status currentUrl pageSource document formattedPage downloadingProcess documentQueue recentDocuments currentAnchorLocation currentUrlIndex backgroundColor bookmark bookDir bindings '
	classVariableNames: 'StartUrl '
	poolDictionaries: ''
	category: 'Network-Web Browser'!

!Scamper commentStamp: '<historical>' prior: 0!
a basic web browser.

It downloads things in a background thread; the background thread puts downloaded objects onto downloadQueue as they arrive.  The queue is checked in the step method.

A custom start page is provided for.  It's not as convenient as bookmarks are, but it does have a lot of flexibility--the user can put anything on the start page that is desired.

There is a hook for displaying the "lint" of a page; currently it's not a very effective linter.!

TestCase subclass: #ScamperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Scamper'!

!ScamperTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class Scamper. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!


!Scamper methodsFor: 'user interface' stamp: 'AK 6/21/2000 17:12'!
loadBookmark
	| directory favorite |
	favorite := 'Bookmark'.
	directory := FileDirectory default.
	(directory directoryExists: favorite)
			ifTrue:[bookDir := directory directoryNamed: favorite.
					self bookmark: (self makeBookmark: bookDir)
					]
			ifFalse:[directory createDirectory: favorite.
					bookDir := directory directoryNamed: favorite.
					self bookmark: Dictionary new ].
	! !

!Scamper methodsFor: 'user interface' stamp: 'AK 6/23/2000 14:32'!
makeBookmark: aDirectory
	| directories dir book extension extension1|
	extension := '*.lin'.
	extension1 := '*.url'. " for IE favorite files"
	book := Dictionary new.
	directories := aDirectory directoryNames.
	directories isEmpty
		ifFalse:[directories do:[:dn |
					dir := aDirectory directoryNamed: dn.
					book add: ( Association key: dn value: ( self makeBookmark: dir))
					]].
	aDirectory fileNames do:
	[:fn | (extension match: fn)
				ifTrue:[ book add: (Association key: (fn truncateTo: (fn size - 4))
							value: (self readUrlFromFile: (aDirectory fullNameFor: fn)))].
		  (extension1 match: fn)
				ifTrue:[ book add: (Association key: (fn truncateTo: (fn size - 4))
							value: ( self importUrl: (aDirectory fullNameFor: fn)))].
	].
	^book
	
		! !

!Scamper methodsFor: 'user interface' stamp: 'dgd 10/28/2003 13:48'!
readUrlFromFile: aFile
	| oldFile url |
	oldFile _ FileStream oldFileOrNoneNamed: aFile.
	oldFile isBinary 
		ifTrue:[ self error: 'not url file' translated]
		ifFalse:[url _ (oldFile contentsOfEntireFile) withBlanksTrimmed.
				url =='' ifTrue:[ self error: 'blank file: url not exist' translated].
				^url asUrl].
	! !

!Scamper methodsFor: 'menus' stamp: 'AK 6/20/2000 15:00'!
addNewSubMenu: aDictionary 
	| subMenu url sub |
	subMenu _ MenuMorph new.
	aDictionary
		keysAndValuesDo: 
			[:name :value | 
			url _ value.
			(url isKindOf: Dictionary)
				ifTrue: 
					[sub _ self addNewSubMenu: url.
					subMenu add: name subMenu: sub]
				ifFalse: [subMenu
						add: name
						target: self
						selector: #jumpToUrl:
						argument: url]].
	^ subMenu! !

!Scamper methodsFor: 'menus' stamp: 'dgd 10/28/2003 13:38'!
addToBookmark
	| key value file filename |
	key _ self document head title ifNil: ['Untitled' translated].
	value _ self currentUrl.
	filename _ key,'.lin'.
	bookDir deleteFileNamed: filename. 
	file _ StandardFileStream fileNamed: (bookDir fullNameFor: filename).
	file ifNil:[self error: 'could not save file' translated].
	file nextPutAll: value asString. 
	file close.
	bookmark add: (Association key: key value: value).
	! !

!Scamper methodsFor: 'menus' stamp: 'sw 9/26/2002 18:21'!
back
	"The user hit the back button -- go to the previous document"

	currentUrlIndex > 1 ifTrue:
		[currentUrlIndex _ currentUrlIndex - 1.
		currentUrlIndex <= recentDocuments size ifTrue:
			[self displayDocument: (recentDocuments at: currentUrlIndex)]]

"this method is added to Scamper: Aibek 4/18/99"! !

!Scamper methodsFor: 'menus' stamp: 'dgd 10/28/2003 13:39'!
bookmark
	| menu sub url |
	menu _ (MenuMorph entitled: ' Bookmark ' translated)
				defaultTarget: self.
	menu addStayUpItem.
	menu addLine.
	menu
		add: 'add to bookmark' translated
		target: self
		selector: #addToBookmark.
	menu add: 'Import...' translated target: self selector: #importBookmark. 
	menu addLine.
	bookmark
		keysAndValuesDo: 
			[:name :value | 
			url _ value.
			(url isKindOf: Dictionary)
				ifTrue: 
					[sub _ self addNewSubMenu: url.
					menu add: name subMenu: sub]
				ifFalse: [menu
						add: name
						selector: #jumpToUrl:
						argument: url]].
	menu popUpInWorld: self currentWorld! !

!Scamper methodsFor: 'menus' stamp: 'dgd 10/28/2003 13:39'!
createBookmarkFiles: aDirectory dict: aDictionary dirname: aName
	| dir file filename |
	(aDirectory directoryExists: aName) ifFalse:[aDirectory createDirectory: aName]. 
	dir _ aDirectory directoryNamed: aName.
	aDictionary keysAndValuesDo:[:k :v |
				(v isKindOf: Dictionary)
					ifTrue:[self createBookmarkFiles: dir dict: v dirname: k]
					ifFalse:[filename _ k, '.lin'.
							dir deleteFileNamed: filename.
							file _ StandardFileStream fileNamed: (dir fullNameFor: filename).
							file ifNil:[self error: 'could not save file' translated].
							file nextPutAll: v asString.
							file close]
				].! !

!Scamper methodsFor: 'menus' stamp: 'dgd 10/28/2003 13:40'!
displayHistory
	"Let the user selecet a previous page to view."

	| menu |
	menu _ MenuMorph entitled: 'Recent URLs' translated.
	menu defaultTarget: self.
	menu addStayUpItem.
	menu addLine.
	recentDocuments reverseDo:
		[:doc |
		menu add: doc url toText selector: #displayDocument: argument: doc].
	menu popUpInWorld: self currentWorld! !

!Scamper methodsFor: 'menus' stamp: 'dgd 10/28/2003 13:40'!
editStartPage
	| win textMorph |
	Smalltalk isMorphic ifFalse: [^ self inform: 'only works for morphic currently' translated].

	win _ SystemWindow labelled: 'edit Bookmark page' translated.
	win model: self.
	textMorph _ PluggableTextMorph on: self text: #startPage  accept: #startPage:.
	win addMorph: textMorph frame: (0@0 extent: 1@1).
	win openInWorld.
	^ true! !

!Scamper methodsFor: 'menus' stamp: 'AM 4/18/1999 14:28'!
forward
	"this method is added to Scamper: Aibek 4/18/99"
	currentUrlIndex >= recentDocuments size
		ifTrue: [^self]
		ifFalse: [
			currentUrlIndex _ currentUrlIndex + 1.
			self displayDocument: (recentDocuments at: currentUrlIndex).
		]
! !

!Scamper methodsFor: 'menus' stamp: 'dgd 10/28/2003 13:41'!
importBookmark
	| newDirectory importLinks filename file |
	newDirectory _ FillInTheBlank request: 'Directory to import' translated initialAnswer: bookDir pathName.
	(newDirectory isNil or: [ newDirectory isEmpty ]) ifTrue: [ ^self ].
	(FileDirectory new directoryExists: newDirectory)
		ifTrue:[importLinks _ self makeBookmark: (FileDirectory on: newDirectory).
				importLinks isEmpty 
					ifFalse:[importLinks associationsDo:[:ass | bookmark add: ass.
							(ass value isKindOf: Dictionary)
								ifTrue:[self createBookmarkFiles: bookDir dict: ass value dirname: ass key]
								ifFalse:[filename _ ass key,'.lin'.
										bookDir deleteFileNamed: filename.
										file _ StandardFileStream fileNamed: (bookDir fullNameFor: filename).
										file ifNil:[self error: 'could not save file' translated].
										file nextPutAll: ass value asString. 
										file close]
								]
							].
				]
		ifFalse:[self importBookmark].! !

!Scamper methodsFor: 'menus' stamp: 'dgd 10/28/2003 13:41'!
importUrl: aFile
	| oldFile url strings position |
	oldFile _ FileStream oldFileOrNoneNamed: aFile.
	oldFile isBinary 
		ifTrue:[ self error: 'not url file' translated]
		ifFalse:[ strings _ (oldFile contentsOfEntireFile) substrings.
				strings do:[:sub |
				( sub includesSubString: 'URL=')
					ifTrue:[ position := sub findString: 'http://'.
							position > 0 ifTrue:[url := sub copyFrom: position to: sub size]
										ifFalse:[ position := sub findString: 'ftp://'.
												position > 0 ifTrue:[url := sub copyFrom: position to: sub size].
												]
							]	
					].
				]. 
	url =='' ifTrue:[ self error: 'blank file: url not exist' translated].
	^url asUrl.! !

!Scamper methodsFor: 'menus' stamp: 'ls 7/14/1998 17:54'!
inspectParseTree
	"inspect a parse tree of the current page's source"
	document ifNotNil: [ document inspect ]! !

!Scamper methodsFor: 'menus' stamp: 'tak 7/7/2004 09:48'!
menu: menu shifted: shifted 
	menu addList: 
#(
('back'	back)
('forward'	forward)
-
('do it (d)'			doIt)
('print it (p)'		printIt)
('inspect it (i)'		inspectIt)
('file it in (G)'		fileItIn)
('debug it'			debugIt)
-
('new URL'	jumpToNewUrl)
('history'	displayHistory)
-
('view source'	viewSource)
('inspect parse tree'	inspectParseTree)
-
('go to start page'	visitStartPage)
('edit start page'	editStartPage)
('bookmark'	bookmark)
('translate'	translateIt)
).

	downloadingProcess
		ifNotNil: 
			[menu addList: #(
				-
				('stop downloading'	stopEverything)
				)].

	^ menu! !

!Scamper methodsFor: 'menus' stamp: 'ls 7/14/1998 02:28'!
perform: selector orSendTo: otherTarget
	"Selector was just chosen from a menu by a user.  If can respond, then
perform it on myself. If not, send it to otherTarget, presumably the
editPane from which the menu was invoked."

	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ otherTarget perform: selector]! !

!Scamper methodsFor: 'menus' stamp: 'kfr 6/11/2000 13:24'!
reload
	self stopEverything.
	self jumpToUrl: currentUrl
	
! !

!Scamper methodsFor: 'menus' stamp: 'dgd 10/28/2003 13:46'!
viewSource
	"view the source HTML of this page"
	(StringHolder new contents: (pageSource withSqueakLineEndings)) openLabel: ('source for {1}' translated format: {currentUrl printString}).! !

!Scamper methodsFor: 'menus' stamp: 'ls 8/12/1998 02:01'!
visitStartPage
	self jumpToAbsoluteUrl: 'browser:start'.
	^true! !

!Scamper methodsFor: 'change/update' stamp: 'ar 11/19/1998 21:37'!
changeAll: aspects
	"We have changed all of the aspects in the given array"
	aspects do:[:symbol| self changed: symbol].! !

!Scamper methodsFor: 'change/update' stamp: 'bolot 2/27/2000 21:05'!
invalidateLayout
	self changeAll: #( backgroundColor formattedPage formattedPageSelection ).! !

!Scamper methodsFor: 'window definition' stamp: 'dgd 10/28/2003 13:47'!
buttonRowPane
	"Create and return a pane of navigation buttons."

	| buttonRow |
	buttonRow _ AlignmentMorph new
		borderWidth: 0;
		layoutInset: 0;
		hResizing: #spaceFill;
		wrapCentering: #center; cellPositioning: #leftCenter;
		clipSubmorphs: true;
		addTransparentSpacerOfSize: (5@0).
	
	buttonRow 
		addMorphBack: (self simpleButtonNamed: 'Back' translated action: #back text: self backButtonText); 
		addTransparentSpacerOfSize: (5@0);
		addMorphBack: (self simpleButtonNamed: 'Forward' translated action: #forward text: self forwardButtonText); 
		addTransparentSpacerOfSize: (5@0);
		addMorphBack: (self simpleButtonNamed: 'History' translated action: #displayHistory text: self historyButtonText); 
		addTransparentSpacerOfSize: (5@0);
		addMorphBack: (self simpleButtonNamed: 'Reload' translated action: #reload text: self reloadButtonText); 
		addTransparentSpacerOfSize: (5@0);
		addMorphBack: (self simpleButtonNamed: 'Home' translated action: #visitStartPage text: self homeButtonText); 
		addTransparentSpacerOfSize: (5@0);
		addMorphBack: (self simpleButtonNamed: 'Stop!!' translated action: #stopEverything text: self stopButtonText); 
		addTransparentSpacerOfSize: (5@0).

	^buttonRow! !

!Scamper methodsFor: 'window definition' stamp: 'ccn 6/28/2000 20:10'!
morphicWindow
	"Create and return a Morphic window for Scamper."

	^(SystemWindow labelled: 'Scamper')
		model: self;
		setProperty: #webBrowserView toValue: true;
		addMorph: self buttonRowPane frame: (0@0 extent: 1@0.07);		"Navigation buttons"
		addMorph: self urlEditPane frame: (0@0.07 extent: 1@0.06);		"URL edit pane"
		addMorph: self webContentsPane frame: (0@0.13 extent: 1@0.81);	"The web page pane"
		addMorph: self statusPane frame: (0@0.94 extent: 1.0@0.06).		"Status pane"! !

!Scamper methodsFor: 'window definition' stamp: 'ccn 6/28/2000 20:53'!
simpleButtonNamed: buttonLabel action: selector text: balloonText
	"Create and return a simple button with Scamper as the target."

	^SimpleButtonMorph new
		label: buttonLabel;
		target: self;
		color: Color transparent;
		cornerStyle: #rounded;
		borderColor: Color black;
		actionSelector: selector;
		actWhen: #buttonUp;
		setBalloonText: balloonText;
		yourself
! !

!Scamper methodsFor: 'window definition' stamp: 'tak 7/5/2004 14:09'!
statusPane
	"Create and return the browser status pane."
	^ (PluggableTextMorph
		on: self
		text: #status
		accept: nil) hideScrollBarIndefinitely; yourself! !

!Scamper methodsFor: 'window definition' stamp: 'tak 7/5/2004 14:07'!
urlEditPane
	"Create and return the URL edit pane."
	^ (PluggableTextMorph
		on: self
		text: #currentUrl
		accept: #jumpToAbsoluteUrl:) acceptOnCR: true;
		 hideScrollBarIndefinitely;
		 yourself! !

!Scamper methodsFor: 'window definition' stamp: 'ccn 6/28/2000 21:00'!
webContentsPane
	"Create and return the web page pane."

	^WebPageMorph
		on: self 
		bg: #backgroundColor 
		text: #formattedPage 
		readSelection: #formattedPageSelection 
		menu: #menu:shifted:! !

!Scamper methodsFor: 'browser urls' stamp: 'dgd 10/28/2003 13:30'!
aboutScamperHTML
	"return a string of HTML which introduces Scamper"
	^'
<html>
<head>
<title>{1}</title>
</head>
<body>
<h1>{2}</h1>
{3}
<ul>
<li><a href="http://minnow.cc.gatech.edu/squeak">{4}</a>
<li><a href="http://minnow.cc.gatech.edu/squeak/14">{5}</a>
<li><a href="http://www.squeak.org">{6}</a>
<li><a href="http://www.squeakland.org">{7}</a>
</ul>
</body>
</html>' format:{
	'About Scamper' translated.
	'Scamper' translated.
	'This is Scamper, a WWW browser for Squeak.  Here are some URL''s to start at:' translated.
	'The Squeak Swiki' translated.
	'Scamper''s Home Page' translated.
	'Squeak''s Home Page' translated.
	'The SqueakLand Home Page' translated
	}! !

!Scamper methodsFor: 'browser urls' stamp: 'ls 6/17/2001 17:57'!
browserUrlContents: aRequest
	aRequest = 'start' ifTrue: [ 
		^MIMEDocument contentType: 'text/html' content: self startPage ].

	aRequest = 'about' ifTrue: [
		^MIMEDocument contentType: 'text/html' content: self aboutScamperHTML ].

	^ nil! !

!Scamper methodsFor: 'access' stamp: 'bolot 2/27/2000 20:59'!
backgroundColor
	^backgroundColor ifNil:
		[self defaultBackgroundColor]! !

!Scamper methodsFor: 'access' stamp: 'AK 6/19/2000 15:39'!
bookmark: aDictionary
	bookmark := aDictionary.! !

!Scamper methodsFor: 'access' stamp: 'ls 7/14/1998 03:18'!
currentUrl
	^currentUrl! !

!Scamper methodsFor: 'access' stamp: 'bolot 2/27/2000 20:59'!
defaultBackgroundColor
	^Color white! !

!Scamper methodsFor: 'access' stamp: 'ls 7/14/1998 02:47'!
document
	"return the current parsed HTML document, or nil if we aren't viewing a page"
	^document! !

!Scamper methodsFor: 'access' stamp: 'ls 7/21/1998 01:16'!
formattedPage
	"format the current page and return it as a Text"
 	^formattedPage ifNil: [ ^Text new ].
	! !

!Scamper methodsFor: 'access' stamp: 'ls 9/10/1998 03:38'!
formattedPageSelection
	currentAnchorLocation ifNil: [ ^0 to: -1 ].
	^currentAnchorLocation to: currentAnchorLocation! !

!Scamper methodsFor: 'access' stamp: 'dgd 10/28/2003 13:27'!
labelString
	"return the title of the current page, or nil if there is none"
	document == nil
		ifTrue: [ ^'Scamper' ]
		ifFalse: [ ^'Scamper: ' , (self document head title ifNil: ['(untitled)' translated]) ]! !

!Scamper methodsFor: 'access' stamp: 'ls 7/16/1998 22:18'!
status
	^status! !

!Scamper methodsFor: 'access' stamp: 'ls 7/16/1998 22:18'!
status: aSymbol
	status _ aSymbol.
	self changed: #status.! !

!Scamper methodsFor: 'start page' stamp: 'dgd 10/28/2003 13:32'!
startPage
	"return the contents of the user's personal start page"
	| file |
	file _ FileStream oldFileOrNoneNamed: 'StartPage.html'.
	file 
		ifNil: [ ^'<title>{1}</title>
<h1>{1}</h1>
{2}' format:{'Personal Start Page' translated. 'This space is empty' translated} ]
		ifNotNil: [ ^file contentsOfEntireFile ]! !

!Scamper methodsFor: 'start page' stamp: 'dgd 10/28/2003 13:47'!
startPage: newPage
	"fill in the contents of the user's personal start page"

	| file |
	FileDirectory default deleteFileNamed: 'StartPage.html'.
	[file _ FileStream fileNamed: 'StartPage.html'.
	file ifNil: [self error: 'could not save file' translated].
	file nextPutAll: newPage asString.
	] ensure: [file close].
	self changed: #startPage.
	^true! !

!Scamper methodsFor: 'changing page' stamp: 'tak 6/23/2004 22:05'!
jumpToAbsoluteUrl: urlText
	"start downloading a new page.  The page source is downloaded in a background thread"
	|  newUrl newSource morph |

	self stopEverything.

	"get the new url"
	newUrl _ urlText asUrl.


	"if it fundamentally doesn't fit the pages-and-contents model used internally, spawn off an external viewer for it"
	newUrl hasContents ifFalse: [ newUrl activate.  ^true ].

	"fork a Process to do the actual downloading, parsing, and formatting.  It's results will be picked up in #step"
	self status: ('downloading {1}...' translated format:{newUrl toText}).

	downloadingProcess _ [ 
	  	newSource _ [ newUrl retrieveContentsForBrowser: self ] ifError: [ :msg :ctx |
			MIMEDocument contentType: 'text/plain' content: msg ].

		newSource 
			ifNil: [ newSource _ MIMEDocument contentType: 'text/plain' content: 'Error retrieving this URL' translated].

			newSource url ifNil: [
				newSource _ MIMEDocument contentType: newSource contentType  content: newSource content  url: newUrl ].

			documentQueue nextPut: newSource.
			downloadingProcess _ nil.
	] newProcess.

	downloadingProcess resume.

	[recentDocuments size > currentUrlIndex] whileTrue: [
		"delete all elements in recentDocuments after currentUrlIndex"
		recentDocuments removeLast.
	].
	currentUrlIndex _ currentUrlIndex + 1.
	morph _ self containingWindow.
	morph ifNotNil: [morph start].

	^true! !

!Scamper methodsFor: 'changing page' stamp: 'dgd 10/28/2003 13:33'!
jumpToNewUrl
	"change to a new, user-specified page"
	| newUrl |
	newUrl _ FillInTheBlank request: 'url to visit' translated initialAnswer: currentUrl toText.
	(newUrl isNil or: [ newUrl isEmpty ]) ifTrue: [ ^self ].
	self jumpToAbsoluteUrl: newUrl! !

!Scamper methodsFor: 'changing page' stamp: 'ls 8/20/1998 11:22'!
jumpToUrl: urlText
	"start downloading a new page.  The page source is downloaded in a background thread"
	self jumpToAbsoluteUrl: (urlText asUrlRelativeTo: currentUrl)
! !

!Scamper methodsFor: 'changing page' stamp: 'ar 11/19/1998 23:12'!
startDownloadingMorphState: morphs
	downloadingProcess _ [
		morphs do: [ :m | m downloadStateIn: self].
	] newProcess.
	downloadingProcess resume.! !

!Scamper methodsFor: 'changing page' stamp: 'ls 8/12/1998 00:56'!
startDownloadingStateIn: aDocument  url: aUrl
	"download the state for the given document in a background thread.  signal the foreground when the data has arrived"
	downloadingProcess _ [	
		aDocument allSubentitiesDo: [ :e |
			e downloadState: aUrl ].
		documentQueue nextPut: #stateDownloaded.
		downloadingProcess _ nil. ] newProcess.
	downloadingProcess resume.! !

!Scamper methodsFor: 'changing page' stamp: 'dgd 10/28/2003 13:25'!
stopEverything
	"stop all background threads and empty queues for communicating with them; bring this Scamper to a sane state before embarking on something new"
	
	downloadingProcess ifNotNil: [
		downloadingProcess terminate.
		downloadingProcess _ nil. ].

	[ documentQueue isEmpty ] whileFalse: [ documentQueue next ].

	self status: 'sittin' translated.! !

!Scamper methodsFor: 'changing page' stamp: 'dgd 10/28/2003 13:35'!
submitFormWithInputs: inputs  url: url  method: method
	"Submit the current form with the given arguments"

	| newUrl newSource | 
	self stopEverything.

	(method asLowercase ~= 'get' and: [ method asLowercase ~= 'post' ]) ifTrue:
		[self inform: ('unknown FORM method: {1}' translated format:{method}).
		^ false ].

	newUrl _ url asUrlRelativeTo: currentUrl.	

	newUrl schemeName ~= 'http' ifTrue:
		[self inform: 'I can only submit forms via HTTP' translated.
		^ false].

	self status: 'submitting form...' translated.

	downloadingProcess _
			[method asLowercase = 'get' 
				ifTrue: [newSource _ newUrl retrieveContentsArgs: inputs] 
				ifFalse: [newSource _ newUrl postFormArgs: inputs].
			documentQueue nextPut:  newSource.

			downloadingProcess _ nil] newProcess.

		downloadingProcess resume.
	^ true
! !

!Scamper methodsFor: 'changing page' stamp: 'dgd 10/28/2003 13:35'!
submitFormWithInputs: inputs url: url method: method encoding: encoding
	"Submit the given form with the provided inputs, url, method, and encoding"

	| newUrl newSource | 
	self stopEverything.

	(method asLowercase ~= 'get' and: [ method asLowercase ~= 'post' ]) ifTrue:
		[self inform: ('unknown FORM method: {1}' translated format:{method}).
		^false ].

	newUrl _ url asUrlRelativeTo: currentUrl.	

	newUrl schemeName ~= 'http' ifTrue:
		[self inform: 'I can only submit forms via HTTP' translated.
		^ false].

	self status: 'submitting form...' translated.

	downloadingProcess _
		[method asLowercase = 'get' 
			ifTrue: [newSource _ newUrl retrieveContentsArgs: inputs] 
			ifFalse:
				[encoding = MIMEDocument contentTypeMultipart
					ifTrue: [newSource _ newUrl postMultipartFormArgs: inputs]
					ifFalse: [newSource _ newUrl postFormArgs: inputs]].
		documentQueue nextPut:  newSource.

		downloadingProcess _ nil] newProcess.

	downloadingProcess resume.
	^ true
! !

!Scamper methodsFor: 'document handling' stamp: 'ff 11/15/2001 23:57'!
displayDocument: mimeDocument
	"switch to viewing the given MIMEDocument"
	|  handled  urlString |
	handled _ false.

	"add it to the history"
"	recentDocuments removeAllSuchThat: [ :d | d url = mimeDocument url ]."
	currentUrlIndex > recentDocuments size
		ifTrue: [recentDocuments addLast: mimeDocument].
"		ifFalse: [recentDocuments removeAt: currentUrlIndex]."
	recentDocuments size > 20 ifTrue: [ recentDocuments removeFirst ].
		
	mimeDocument mainType = 'image' 
		ifTrue: [handled _ self displayImagePage: mimeDocument].

	(mimeDocument contentType beginsWith: 'text/html') 
		ifTrue: [handled _ self displayTextHtmlPage: mimeDocument].

	mimeDocument contentType = 'x-application/shockwave-flash'
		ifTrue:[handled _ self displayFlashPage: mimeDocument].

	(#('audio/midi' 'audio/x-midi') includes: mimeDocument contentType) 
		ifTrue: [handled _ self processMidiPage: mimeDocument].

	"Before we display plain text files we should look at the extension of the URL"
	(handled not and:[ mimeDocument mainType = 'text']) ifTrue:[
		urlString _ mimeDocument url toText.
		(urlString endsWithAnyOf: #('.gif' '.jpg' '.pcx')) 
			ifTrue:[handled _ self displayImagePage: mimeDocument].
		(handled not and:[urlString endsWithAnyOf: #('.mid' '.midi')])
			ifTrue:[handled _ self processMidiPage: mimeDocument].
		(handled not and:[urlString endsWith: '.swf'])
			ifTrue:[handled _ self displayFlashPage: mimeDocument].
	].

	(handled not and: [ mimeDocument mainType = 'text']) ifTrue: [
		self displayPlainTextPage: mimeDocument.
		handled _ true].


	handled ifFalse: [self processUnhandledPage: mimeDocument].! !

!Scamper methodsFor: 'document handling' stamp: 'dgd 10/28/2003 13:22'!
displayFlashPage: newSource
	"A shockwave flash document -- embed it in a text"
	| attrib stream player |
	stream _ (RWBinaryOrTextStream with: newSource content) binary reset.
	(FlashFileReader canRead: stream) ifFalse:[^false]. "Not a flash file"
	player _ (FlashMorphReader on: stream) processFileAsync.
	player sourceUrl: newSource url.
	player startPlaying.
	attrib _ TextAnchor new anchoredMorph: player.
	formattedPage _ (Character value: 1) asText.
	backgroundColor _ self defaultBackgroundColor.
	formattedPage addAttribute: attrib from: 2 to: 2.

	currentUrl _ newSource url.
	pageSource _ newSource content.

	"remove it from the history--these thigns are too big!!"
	"ideally, there would be a smarter history mechanism that can do things like remove items when memory consumption gets too high...."
"	recentDocuments removeLast."

	self changeAll: 	#(currentUrl relabel hasLint lint backgroundColor formattedPage formattedPageSelection).
	self status: 'sittin' translated.
	^true! !

!Scamper methodsFor: 'document handling' stamp: 'dgd 10/28/2003 13:22'!
displayImagePage: newSource
	"an image--embed it in a text"
	| image imageMorph attrib text handled |
	handled _ true.
	backgroundColor _ self defaultBackgroundColor.
	formattedPage _ [
		image _ ImageReadWriter formFromStream: (RWBinaryOrTextStream with: newSource content) binary reset.
		imageMorph _ ImageMorph new image: image.
		attrib _ TextAnchor new anchoredMorph: imageMorph.
		text _ (Character value: 1) asText.
		text addAttribute: attrib from: 2 to: 2.
		text] ifError: [ :msg :ctx | handled _ false ].

	currentUrl _ newSource url.
	pageSource _ newSource content.

	"remove it from the history--these thigns are too big!!"
	"ideally, there would be a smarter history mechanism that can do things like remove items when memory consumption gets too high...."
"	recentDocuments removeLast."

	self changeAll: 	#(currentUrl relabel hasLint lint backgroundColor formattedPage formattedPageSelection).
	self status: 'sittin' translated.
	^handled! !

!Scamper methodsFor: 'document handling' stamp: 'dgd 10/28/2003 13:22'!
displayPlainTextPage: newSource
	"treat as plain text"
	pageSource _ newSource content.
	document _ nil.
	formattedPage _ pageSource withSqueakLineEndings.
	backgroundColor _ self defaultBackgroundColor.
	currentUrl _ newSource url.

	self status: 'sittin' translated.
	self changeAll: 	#(currentUrl relabel hasLint lint formattedPage formattedPage
formattedPageSelection).
	^true! !

!Scamper methodsFor: 'document handling' stamp: 'tak 6/23/2004 22:50'!
displayTextHtmlPage: newSource
	"HTML page--format it"
	| formatter bgimageUrl bgimageDoc bgimage |
	currentUrl _ newSource url.
	pageSource _ newSource content.
	self status: 'parsing...' translated.
	document _ HtmlParser parse: (ReadStream on: pageSource).
	document _ HtmlParser parse: document asHtmlAfterScripts.
	document replaceAllTextsToMultiString.
	self status: 'laying out...' translated.
	formatter _ HtmlFormatter preferredFormatterClass new.
	formatter browser: self.
	formatter baseUrl: currentUrl.
	document addToFormatter: formatter.

	formattedPage _ formatter text.
	(bgimageUrl _ document body background)
		ifNotNil:
			[bgimageDoc _ (bgimageUrl asUrlRelativeTo: currentUrl) retrieveContents.
			[bgimage _ ImageReadWriter formFromStream: bgimageDoc contentStream binary]
				ifError: [:err :rcvr | "ignore" bgimage _ nil]].
	bgimage
		ifNotNil: [backgroundColor _ bgimage]
		ifNil: [backgroundColor _ Color fromString: document body bgcolor].
	currentUrl fragment
		ifNil: [ currentAnchorLocation _ nil ]
		ifNotNil: [ currentAnchorLocation _
				formatter anchorLocations 
					at: currentUrl fragment asLowercase
					ifAbsent: [ nil ] ].

	self startDownloadingMorphState: (formatter incompleteMorphs).

	self changeAll: 	#(currentUrl relabel hasLint lint backgroundColor formattedPage formattedPageSelection).
	self status: 'done.' translated.
	"pardon this horrible hack...(tk)"
	(currentUrl authority beginsWith: 'ets.freetranslation.com') ifTrue: [
		self status: 'done.
**** Please Scroll Down To See Your Results ****' translated].
	^true! !

!Scamper methodsFor: 'document handling' stamp: 'dgd 10/28/2003 13:24'!
processMidiPage: newSource
	Smalltalk at: #MIDIFileReader ifPresent:
		[:reader |
		reader playStream: (RWBinaryOrTextStream with: newSource content) reset binary.
		self status: 'sittin' translated.
		^true].
	^false! !

!Scamper methodsFor: 'document handling' stamp: 'dgd 10/28/2003 13:38'!
processUnhandledPage: newSource
	"offer to save it to a file"
	| fileName file |
	self status: 'sittin' translated.

	(newSource url toText endsWith: '.pr') ifTrue: [
		(self confirm: 'Looks like a Squeak project - do you want to load it as such?' translated) ifTrue: [
			^ProjectLoading thumbnailFromUrl: newSource url toText
		].
	].

	(self confirm: ('unkown content-type {1}--
Would you like to save to a file?' translated format:{newSource contentType})) ifFalse: [ ^false ].

	fileName _ ''.
	[
		fileName _ FillInTheBlank request: 'file to save in' translated initialAnswer: fileName.
		fileName isEmpty ifTrue: [ ^self ].
		file _ FileStream fileNamed: fileName.
		file == nil
	] whileTrue.

	file reset.
	file binary.
	file nextPutAll: newSource content.
	file close.
	^true! !

!Scamper methodsFor: 'stepping' stamp: 'dgd 10/28/2003 13:25'!
step
	"check if a new document has arrived"
	| results |
	[documentQueue isEmpty] whileFalse: [
		results _ documentQueue next.

		results == #stateDownloaded ifTrue: [ 
			"images and such have been downloaded; update the page"
			self status: 'reformatting page...' translated.
			formattedPage _ document formattedTextForBrowser: self defaultBaseUrl: currentUrl.
			backgroundColor _ Color fromString: document body bgcolor.
			self changeAll: #(backgroundColor formattedPage).
			self status: 'sittin' translated. ]
		 ifFalse: [		
			self displayDocument: results 	
		] ]! !

!Scamper methodsFor: 'stepping' stamp: 'di 1/14/1999 09:00'!
wantsSteps
	^ true! !

!Scamper methodsFor: 'button text' stamp: 'dgd 10/28/2003 13:20'!
backButtonText
	^ 'Go back to previous URL in history' translated! !

!Scamper methodsFor: 'button text' stamp: 'dgd 10/28/2003 13:20'!
forwardButtonText
	^ 'Go forward to next URL in history' translated! !

!Scamper methodsFor: 'button text' stamp: 'dgd 10/28/2003 13:20'!
historyButtonText
	^ 'Return to a recent URL in history' translated! !

!Scamper methodsFor: 'button text' stamp: 'dgd 10/28/2003 13:20'!
homeButtonText
	^ 'Go to start page' translated! !

!Scamper methodsFor: 'button text' stamp: 'dgd 10/28/2003 13:20'!
reloadButtonText
	^ 'Reload page' translated! !

!Scamper methodsFor: 'button text' stamp: 'dgd 10/28/2003 13:20'!
stopButtonText
	^ 'Stop loading page' translated! !

!Scamper methodsFor: 'initialization' stamp: 'dgd 10/28/2003 13:23'!
initialize
	documentQueue _ SharedQueue new.
	recentDocuments _ OrderedCollection new.
	bookmark := Dictionary new.
	currentUrlIndex _ 0.
	currentUrl _ ''.
	pageSource _ ''.
	document _ HtmlParser parse: (ReadStream on: '').
	self status: 'sittin' translated! !

!Scamper methodsFor: 'testing' stamp: 'ls 7/14/1998 21:44'!
isWebBrowser
	^true! !

!Scamper methodsFor: 'not yet categorized' stamp: 'ls 9/14/1998 20:15'!
doItContext
	^nil! !

!Scamper methodsFor: 'not yet categorized' stamp: 'tak 5/12/2004 16:17'!
doItReceiver
	^self! !

!Scamper methodsFor: 'not yet categorized' stamp: 'ls 7/29/1998 03:28'!
release
	self stopEverything.
	super release.! !

!Scamper methodsFor: 'creation' stamp: 'ccn 6/28/2000 19:52'!
openAsMorph

	^self morphicWindow openInWorld
! !

!Scamper methodsFor: 'lint' stamp: 'ls 8/1/1998 03:10'!
hasLint
	"whether the current page has any questionable HTML in it"
	document ifNil: [ ^false ].
	^document lint ~= ''! !

!Scamper methodsFor: 'lint' stamp: 'ls 9/14/1998 20:38'!
lint
	"return a string describing any questionable HTML that was noticed in the current page"
	"(not currently very comprehensive)"
	document ifNil: [ ^'' ].
	^document lint! !

!Scamper methodsFor: 'lint' stamp: 'ls 7/29/1998 00:07'!
showLint
	(StringHolder new contents: self lint) openLabel: 'lint for ', self currentUrl printString.! !


!Scamper class methodsFor: 'instance creation' stamp: 'sd 2/6/2002 21:36'!
fileReaderServicesForFile: fullName suffix: suffix


	^(suffix = 'htm') | (suffix = 'html') 
		ifTrue: [self services]
		ifFalse: [#()]! !

!Scamper class methodsFor: 'instance creation' stamp: 'sma 4/30/2000 11:33'!
new
	^ super new initialize! !

!Scamper class methodsFor: 'instance creation' stamp: 'sw 6/21/2001 17:54'!
newInstancePointingAt: aStringOrUrl
	"Answer a Scamper browser on specified url.
		Scamper newInstancePointingAt: 'www.squeak.org'
		Scamper newInstancePointingAt: 'file://C%3A/test.htm'
	"

	^ (self new
		jumpToUrl: aStringOrUrl asUrl) morphicWindow applyModelExtent! !

!Scamper class methodsFor: 'instance creation' stamp: 'sw 6/21/2001 17:52'!
newOpenableMorph
	"Answer a morph that bears the receiver, but don't blast it onto the screen"

	^ self newInstancePointingAt: StartUrl! !

!Scamper class methodsFor: 'instance creation' stamp: 'tk 7/14/2000 15:20'!
newOrExistingOn: aStringOrUrl
	| aUrl siteStr |
	"If a Scamper is open on the same site, return its SystemWindow, else return a new Scamper."

aUrl _ aStringOrUrl asUrl.
siteStr _ aUrl schemeName, '://', aUrl authority.

Smalltalk isMorphic ifTrue: [
	World submorphsDo: [:m | 
		((m isKindOf: SystemWindow) and: [m model isKindOf: Scamper]) ifTrue: [
			(m model currentUrl asString beginsWith: siteStr) ifTrue: [
					m expand.
					^ m]]]].

^ self new openAsMorph
! !

!Scamper class methodsFor: 'instance creation' stamp: 'sma 4/30/2000 11:48'!
open
	"Scamper open"

	^ self openOnUrl: StartUrl! !

!Scamper class methodsFor: 'instance creation' stamp: 'sma 4/30/2000 11:48'!
openAsMorph
	"Deprecated. Use open instead."

	^ self open! !

!Scamper class methodsFor: 'instance creation' stamp: 'hg 8/3/2000 17:21'!
openFile: fullFileName
	Scamper openOnUrl: 
		(FileDirectory forFileName: fullFileName) url , 
		(FileDirectory localNameFor: fullFileName) encodeForHTTP
! !

!Scamper class methodsFor: 'instance creation' stamp: 'SD 11/14/2001 22:17'!
openFileFromFileList: fullName
	
	self openFile: fullName! !

!Scamper class methodsFor: 'instance creation' stamp: 'sma 4/30/2000 11:49'!
openOnUrl: aStringOrUrl
	"Open Scamper browser on specified url.
		Scamper openOnUrl: 'www.squeak.org'
		Scamper openOnUrl: 'file://C%3A/test.htm'
	"

	^ self new
		jumpToUrl: aStringOrUrl asUrl;
		openAsMorph! !

!Scamper class methodsFor: 'instance creation' stamp: 'sw 2/17/2002 02:45'!
serviceOpenInWebBrowser
	"Answer a service for opening a web browser on a file"

	^ SimpleServiceEntry 
			provider: self 
			label: 'open in web browser'
			selector: #openFile:
			description: 'open in web browser'
			buttonLabel: 'open'! !

!Scamper class methodsFor: 'instance creation' stamp: 'sd 2/1/2002 22:20'!
services

	^ Array with: self serviceOpenInWebBrowser

! !

!Scamper class methodsFor: 'class initialization' stamp: 'ads 3/29/2003 13:25'!
unload
	"Unload the receiver from global registries"

	FileList unregisterFileReader: self.
	TheWorldMenu unregisterOpenCommandWithReceiver: self.
	Flaps unregisterQuadsWithReceiver: self.
	WebBrowser unregister: self.! !

!Scamper class methodsFor: 'initialization' stamp: 'sma 4/30/2000 11:39'!
StartUrl: aStringOrUrl
	"Specify the default URL to start from."

	StartUrl _ aStringOrUrl asUrl! !

!Scamper class methodsFor: 'initialization' stamp: 'ads 4/2/2003 18:53'!
initialize
	"Initialize the class"

	self StartUrl: 'browser:about'.
	FileList registerFileReader: self.
	self registerInOpenMenu.
	WebBrowser register: self.
	
	Flaps registerQuad: { #Scamper. #newOpenableMorph. 'Scamper'. 'A web browser' } forFlapNamed: 'Tools'.! !

!Scamper class methodsFor: 'initialization' stamp: 'ads 4/2/2003 18:52'!
registerInOpenMenu
	"Register the receiver in the system's open menu"

	TheWorldMenu registerOpenCommand: 
		{ 'web browser' . { Scamper . #openAsMorph }. '"Scamper", a web browser' }.
! !


!ScamperTest methodsFor: 'initialize-release' stamp: 'md 4/16/2003 15:08'!
testOpen

	|scamper|
		
	self shouldnt: [scamper := Scamper open] raise: Error.
	scamper delete.
! !

Scamper initialize!
